/**
 * @author Ilya Peresadin <pva701@gmail.com>
 * @created 22.11.16
 */
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

/**
 * @author Ilya Peresadin <pva701@gmail.com>
 * @created 11.10.16
 */
public class TestGen {

    private final boolean FOR_POLYGON = true;

    public static final int MAX_MOD = 1_000_000_000;

    int testCount;
    static Random random = new Random(0x101);

    private String folder;
    boolean noModifyTest = false;

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void println(PrintWriter writer, int... arr) {
        for (int i = 0; i < arr.length; ++i)
            if (i + 1 == arr.length) writer.println(arr[i]);
            else writer.print(arr[i] + " ");
    }

    public void print(String n, int m, String descr) throws IOException {
        String path = (FOR_POLYGON ?
                String.format("%d",++testCount) :
                String.format("../%s/%02d", folder, ++testCount));

        PrintWriter out = new PrintWriter(path);
        System.out.println("Generating test #" + testCount + ": " + n + " " + m + ", " + descr);
        out.println(n + " " + m);
        out.close();
    }

    public void genSamples() throws IOException {
        setFolder("tests");
        noModifyTest = true;
        System.out.println("==== Generating samples ===");
        print("3?1?", 3215, "sample1");
        print("??", 20, "sample2");
        print("?1?", 730, "sample3");
        noModifyTest = false;
    }

    void genHandleTests() throws IOException {
        noModifyTest = true;
        System.out.println("==== Generating handle tests ====");
        print("0", 1, "");
        print("0", 10, "");
        print("?", 10, "");
        print("??", 100, "");
        print("30", 99, "");
        print("1", 5, "");
        noModifyTest = false;
    }

    void genTests() throws IOException {
        System.out.println("==== Generating tests ====");
        for (int l = 6; l <= 14; ++l)
            for (int it = 1; it <= 5; ++it) {
                int q = randInt(0, l);
                genRandom(l, q, (int)Math.min(MAX_MOD, pow10(l)));
            }

        print("??????????????", 999999999, "");
        print("??????????????", 123456789, "");
        print("??????????????", 23456789, "");
        print("???????1234567", 123456789, "");
        print("???????1234567", 999999999, "");
        print("???????12?4?5?", 999999999, "");
        print("???????12?4?5?", 1, "");
        print("???????12?4?5?", 99, "");
        print("???????12?4567", 123456789, "");
        print("???????0??4???", 9999997, "");
        print("???????0??4???", 29999997, "");
        print("????????542389", 911111111, "");
    }


    private void genRandom(int l, int q, int maxMod) throws IOException {
        int x = 1;
        for (int i = 1; i <= l; ++i) {
            x *= 10;
        }
        int[] part = genCombine(l, q);
        char[] num = new char[l];
        for (int e : part) num[e - 1] = '?';
        for (int i = 0; i < l; ++i)
            if (num[i] == 0) {
                char d = (char)((i == 0 ? randInt(1, 9) : randInt(0, 9)) + '0');
                num[i] = d;
            }
        String n = new String(num);
        int m = randInt(1, maxMod);
        print(n, m, "random test");
    }

    public void run() throws IOException {
        if (!FOR_POLYGON) {
            new File("../tests").mkdir();
        }

        genSamples();
        genHandleTests();
        genTests();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }

    private static int randInt(int l, int r) {
        return random.nextInt(r - l + 1) + l;
    }

    private void shuffle(List<?> a) {
        Collections.shuffle(a, random);
    }

    private int[] genCombine(int n, int k) {
        if (k > n)
            throw new IllegalArgumentException("k > n");

        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            p.add(i + 1);
        shuffle(p);
        int[] ret = new int[k];
        List<Integer> sublist = p.subList(0, k);
        for (int i = 0; i < k; ++i)
            ret[i] = sublist.get(i);
        Arrays.sort(ret);
        return ret;
    }

    private long pow10(int x) {
        long r = 1;
        for (int i = 1; i <= x; ++i)
            r *= 10;
        return r;
    }
}

