import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

/**
 * @author Ilya Peresadin <pva701@gmail.com>
 * @created 11.10.16
 */
public class TestGen {
    private final boolean FOR_POLYGON = true;

    public static final int MAX_N = 100_000;

    int testCount;
    static Random random = new Random(0x101);

    private String folder;
    boolean noModifyTest = false;

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void println(PrintWriter writer, int... arr) {
        for (int i = 0; i < arr.length; ++i)
            if (i + 1 == arr.length) writer.println(arr[i] + 1);
            else writer.print(1 + arr[i] + " ");
    }

    public void print(int[] a, String descr) throws IOException {
        String path = (FOR_POLYGON ?
                String.format("%d",++testCount) :
                String.format("../%s/%02d", folder, ++testCount));

        PrintWriter out = new PrintWriter(path);
        System.out.println("Generating test #" + testCount + ": " + descr);
        int n = a.length;
        out.println(n);
        println(out, a);
        out.close();
    }

    public void genSamples() throws IOException {
        setFolder("tests");
        noModifyTest = true;
        System.out.println("==== Generating samples ===");
        print(new int[]{0, 1, 2}, "sample1");
        print(new int[]{0, 2, 0}, "sample2");
        print(new int[]{1, 2, 3, 0}, "sample3");
        noModifyTest = false;
    }

    void genHandleTests() throws IOException {
        noModifyTest = true;
        System.out.println("==== Generating handle tests ====");
        print(new int[]{3, 0, 4, 6, 5, 2, 5}, "n=7");
        noModifyTest = false;
    }

    void genTests() throws IOException {
        System.out.println("==== Generating tests ====");
        printYes(6, 1);
        printYes(10, 1);
        printNo(6, 4, 2);
        printYes(MAX_N, 1);
        printYes(MAX_N, 1);
        printYes(MAX_N, MAX_N / 2);
        printYes(MAX_N, 2);
        printYes(MAX_N, 10);
        printYes(MAX_N, MAX_N / 3);
        printYes(MAX_N, MAX_N - 1);
        printYes(MAX_N, MAX_N);

        printNo(MAX_N, 1, MAX_N - 1);
        printNo(MAX_N, 2, MAX_N  / 2);
        printNo(MAX_N, 2, MAX_N  - 2);
        printNo(MAX_N, MAX_N - 2, 2);
        printNo(MAX_N, MAX_N / 2, MAX_N / 2);
        printNo(MAX_N, MAX_N / 2, 1000);
        printNo(MAX_N, MAX_N - 1, 1);

        for (int i = 0; i < 5; ++i) {
            printRandom(MAX_N);
        }
    }

    private void printRandom(int n) throws IOException {
        int[] a = genPermutation(n);
        print(a, "Random test");
    }

    private void printYes(int n, int cycle) throws IOException {
        int[] p = genPermutation(n);
        int[] a = new int[n];
        for (int i = 0; i + 1 < cycle; ++i) {
            a[p[i]] = p[i + 1];
        }
        a[p[cycle - 1]] = p[0];
        int c = randInt(0, cycle - 1);
        for (int i = cycle; i + 1 < n; ++i) {
            a[p[i]] = p[i + 1];
        }
        if (cycle != n) {
            a[p[n - 1]] = p[c];
        }

        if (n == cycle) {
            print(a, "Cycle from n vertexes, answer NO");
        } else {
            print(a, "Answer YES");
        }
    }

    private void printNo(int n, int cycle, int parts) throws IOException {
        int[] p = genPermutation(n);
        int[] a = new int[n];
        for (int i = 0; i + 1 < cycle; ++i) {
            a[p[i]] = p[i + 1];
        }
        a[p[cycle - 1]] = p[0];
        if (n - cycle < parts) {
            throw new IllegalArgumentException("n - cycle < parts");
        }
        int[] comb = genCombine(n - cycle - 1, parts - 1);
        int fst = (comb.length == 0 ? n : comb[0] + cycle);
        addBranch(randInt(0, cycle - 1), p, cycle, fst, a);
        for (int i = 0; i < comb.length; ++i) {
            int r = (i + 1 == comb.length ? n : comb[i + 1] + cycle);
            addBranch(randInt(0, cycle - 1), p, comb[i] + cycle, r, a);
        }
        print(a, "Length of cycle = " + cycle + ", with " + parts + " branches, answer NO");
    }

    private void addBranch(int root, int[] p, int l, int r, int[] a) {
        for (int i = l; i + 1 < r; ++i) {
            a[p[i]] = p[i + 1];
        }
        a[p[r - 1]] = p[root];
    }

    public void run() throws IOException {
        if (!FOR_POLYGON) {
            new File("../tests").mkdir();
        }

        genSamples();
        genHandleTests();
        genTests();
    }

    private void swap(int[] a, int i, int j) {
        int t = a[i];
        a[i] = a[j];
        a[j] = t;
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }

    private int[] genCombine(int n, int k) {
        if (k > n)
            throw new IllegalArgumentException("k > n");

        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            p.add(i + 1);
        shuffle(p);
        int[] ret = new int[k];
        List<Integer> sublist = p.subList(0, k);
        for (int i = 0; i < k; ++i)
            ret[i] = sublist.get(i);
        Arrays.sort(ret);
        return ret;
    }

    private int[] genPartitionViaCombine(int n, int[] comb) {
        int k = comb.length + 1;
        int[] ret = new int[k];
        if (k >= 2)
            ret[0] = comb[0];

        for (int i = 1; i + 1 < k; ++i)
            ret[i] = comb[i] - comb[i - 1];
        ret[k - 1] = k >= 2 ?  n - comb[k - 2] : n;
        return ret;
    }

    public int[] genPartition(int sum, int k) {
        int[] comb = genCombine(sum - 1, k - 1);
        return genPartitionViaCombine(sum, comb);
    }

    private int[] genPartitionWithZeroes(int sum, int k) {
        int[] comb = genCombineWithEq(sum, k - 1);
        return genPartitionViaCombine(sum, comb);
    }

    private int[] genCombineWithEq(int n, int k) {
        int[] ret = new int[k];
        for (int i = 0; i < k; ++i)
            ret[i] = randInt(0, n);
        Arrays.sort(ret);
        return ret;
    }

    private int[] genPermutation(int n) {
        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i) {
            p.add(i);
        }
        Collections.shuffle(p, random);
        int[] ret = new int[n];
        for (int i = 0; i < n; ++i) {
            ret[i] = p.get(i);
        }
        return ret;
    }

    private void shuffle(List<?> a) {
        Collections.shuffle(a, random);
    }

    private void shuffle(int[] a, int from, int to) {
        int[] b = Arrays.copyOfRange(a, from, to);
        Integer[] bInteger = new Integer[b.length];
        for (int i = 0; i < b.length; ++i)
            bInteger[i] = b[i];

        List<Integer> arr = Arrays.asList(bInteger);
        shuffle(arr);
        for (int i = from; i < to; ++i)
            a[i] = arr.get(i - from);
    }

    private int[] toIntArray(Integer[] a) {
        int[] ret = new int[a.length];
        for (int i = 0; i < ret.length; ++i)
            ret[i] = a[i];
        return ret;
    }


    private static int randInt(int l, int r) {
        return random.nextInt(r - l + 1) + l;
    }

    private static int randInt(int n) {
        return random.nextInt(n) + 1;
    }
}
