import java.io.*;
import java.util.Arrays;
import java.util.StringTokenizer;

/**
 * @author Ilya Peresadin <pva701@gmail.com>
 * @created 27.09.16
 */
public class numrecovery_ip_int implements Closeable {

    private static final int MAX_MOD = 1_000_000_000;
    private static final int MAX_LEN = 14;
    public static final int BITS = 32;
    public static final int LOG_BITS = 5;

    FastScanner in;
    PrintWriter out;

    public numrecovery_ip_int() throws IOException {
        in = new FastScanner(new File("numrecovery.in"));
        out = new PrintWriter("numrecovery.out");
    }

    @Override
    public void close() throws IOException {
        out.close();
    }

    private int pow10(int x) {
        int ret = 1;
        for (int i = 1; i <= x; ++i) {
            ret *= 10;
        }
        return ret;
    }

    private int cntQ(String s) {
        int ret = 0;
        for (int i = 0; i < s.length(); ++i)
            if (s.charAt(i) == '?') {
                ret++;
            }
        return ret;
    }

    private int cntWays(String str, boolean isRig) {
        if (str.length() != 0 && str.charAt(0) == '?' && isRig) {
            return pow10(cntQ(str.substring(1))) * 9;
        } else {
            return pow10(cntQ(str));
        }
    }

    private void setBit(int[] a, int bit) {
        a[bit>>LOG_BITS] |= 1L<<(bit & (BITS - 1));
    }

    private boolean isBit(int[] a, int bit) {
        int r = (a[bit>>LOG_BITS]>>>(bit & (BITS - 1)))&1;
        if (r == 1) return true;
        return false;
    }

    private int[] lSort(int[] a, int mod) {
        int maxVal = Math.min(mod, pow10(MAX_LEN / 2));
        int[] masks = new int[maxVal / BITS + 1];
        for (int x : a) {
            setBit(masks, x);
        }
        int[] ret = new int[maxVal];
        int lsz = 0;
        for (int i = 0; i < maxVal; ++i)
            if (isBit(masks, i)) {
                ret[lsz++] = i;
            }
        return Arrays.copyOfRange(ret, 0, lsz);
    }

    private int[] rSort(int[] a, int pw, int mod) {
        long l1t = System.currentTimeMillis();
        int[] masks = new int[MAX_MOD / BITS + 1];
        for (int x : a) {
            setBit(masks, (int) ((long) x * pw % mod));
        }
        int sz = 1<<16;
        int[] rightmostBit = new int[sz];
        rightmostBit[0] = -1;
        rightmostBit[1] = 0;
        for (int i = 1; i < sz; ++i) {
            if (2 * i < sz) {
                rightmostBit[2 * i] = rightmostBit[i] + 1;
            }
            if (2 * i + 1 < sz) {
                rightmostBit[2 * i + 1] = 0;
            }
        }

        int rsz = 0;
        int[] ret = new int[a.length];
        for (int i = 0; i < masks.length; ++i) {
            if (masks[i] == 0) continue;
            long m = masks[i];
            int b = 0;
            while (b < BITS) {
                int cr = (int)(m & (sz - 1));
                if (rightmostBit[cr] == -1) {
                    b += 16;
                    m >>>= 16;
                } else {
                    int locBit = b + rightmostBit[cr];
                    int num = i * BITS + locBit;
                    ret[rsz++] = num;
                    b += rightmostBit[cr] + 1;
                    m >>>= (rightmostBit[cr] + 1);
                }
            }
        }
        long r1t = System.currentTimeMillis();
        System.out.println("rsort = " + (r1t - l1t) / 1000.0);
        return Arrays.copyOfRange(ret, 0, rsz);
    }

    private String str;
    private int strlength;
    private boolean isRig;
    private int[] ret;
    private int sz;
    private int mod;

    private int[] brute(String str, int m, boolean isRig) {
        this.isRig = isRig;
        this.str = str;
        this.strlength = str.length();
        this.ret = new int[cntWays(str, isRig)];
        this.sz = 0;
        this.mod = m;
        recBrute(0, 0);
        return ret;
    }

    private void recBrute(int pos, int val) {
        if (pos == strlength) {
            ret[sz++] = val % mod;
            return;
        }
        if (str.charAt(pos) == '?') {
            for (int d = (isRig && pos == 0 ? 1 : 0); d < 10; ++d) {
                recBrute(pos + 1, val * 10 + d);
            }
        } else {
            recBrute(pos + 1, val * 10 + (str.charAt(pos) - '0'));
        }
    }

    public void run() {
        long lt = System.currentTimeMillis();
        String s = in.next();
        int m = in.nextInt();
        int n = s.length();
        int[] rig = brute(s.substring(0, n / 2), m, true);
        int[] lef = brute(s.substring(n / 2, n), m, false);
        int pw = pow10((n + 1) / 2);
        lef = lSort(lef, m);
        rig = rSort(rig, pw % m, m);

        int mn = (lef[0] + rig[0]) % m, cr;
        System.out.println("START = " + mn);
        for (int l = 0, r = rig.length - 2; l < lef.length; ++l) {
            for (; r >= 0 && lef[l] + rig[r] >= m; --r);
            cr = (lef[l] + rig[r + 1]) % m;
            if (cr < mn) {
                mn = cr;
            }
        }
        out.println(mn);
        long rt = System.currentTimeMillis();
        System.out.println("Total = " + (rt - lt) / 1000.0);
    }

    public static void main(String[] args) throws IOException {
        numrecovery_ip_int instance = new numrecovery_ip_int();
        instance.run();
        instance.close();
    }

    class FastScanner {
        BufferedReader br;
        StringTokenizer st;

        FastScanner(File f) {
            try {
                br = new BufferedReader(new FileReader(f));
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
        }

        String next() {
            while (st == null || !st.hasMoreTokens()) {
                try {
                    st = new StringTokenizer(br.readLine());
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            return st.nextToken();
        }

        int nextInt() {
            return Integer.parseInt(next());
        }
    }
}
